#include "ProToolkit.h"
#include "ProMenu.h"
#include "ProMenuBar.h"
#include "ProSolid.h"
#include "ProMechanica.h"
#include "ProMechGeometry.h"
#define PT_TEST_LOG(func,status,err)                      \
  if (err)                                                \
    {                                                     \
      fprintf (logfile, "Error: %s\t%d\n", func, status); \
    }                                                     \
  else                                                    \
    fprintf (logfile, "Success: %s\t\n", func);

#define print_status(func)                              \
  PT_TEST_LOG (func, status, status != PRO_TK_NO_ERROR)


/* Globals */

FILE *logfile;

/* Prototypes */

int toolkit_issue (ProAppData arg_1, int arg_2);

void errstatus(ProError err, char* name);

static int numberOfFaces;

ProError faceIter(ProMechSurface f, ProError pe, ProAppData data)
{
  int face_id;
  ProMechsurfaceIdGet(f,&face_id);
  fprintf(logfile,"Visiting face: %x id: %d\n",f,face_id);
  numberOfFaces = numberOfFaces+1;
  return PRO_TK_NO_ERROR;
}

ProError solidIter(ProMechSolid s, ProError pe, ProAppData data) 
{
  fprintf(logfile, "Visiting solid: %x\n",s);
  pe = ProMechsolidSurfaceVisit(s,faceIter,NULL,NULL);
  return PRO_TK_NO_ERROR;
}

int user_initialize()
{	
  uiCmdCmdId   cmd_id;

  ProCmdActionAdd("Toolkit_Issue", (uiCmdCmdActFn) toolkit_issue, uiProe2ndImmediate,
                  NULL, PRO_B_TRUE, PRO_B_TRUE, &cmd_id);
  ProMenubarMenuAdd("Toolkit Menu", "Toolkit Menu", "Help", PRO_B_TRUE, L"message.txt");
  ProMenubarmenuPushbuttonAdd("Toolkit Menu", "Toolkit Button", "Toolkit Button", "Toolkit Button",
                              NULL, PRO_B_TRUE, cmd_id, L"message.txt");

  return (0);
}

int toolkit_issue (ProAppData arg_1, int arg_2)
{
  ProError proerror, status;
  ProMdl model;
  //==========================================================================
  ProName mname;
  ProMdl tempModel;
  ProMdlType mtype;
  ProVector ept, pnt;
  char *s1, *s2;
  ProMechModel mModel;
  ProMechModelStatus mStatus;
  int mid;
  int modelCount = 0, aflag;
  ProMatrix trans;
  ProMechModelData mdata;
  ProSolid temp1Model;
  logfile = fopen("log.txt", "w");
  status=ProMdlCurrentGet(&tempModel);
  print_status("ProMdlCurrentGet");

  // Get Model Name
  status=ProMdlNameGet(tempModel, mname);
  print_status("ProMdlNameGet");

  // Get Model Type

  /*  if (ProMdlTypeGet(tempModel, &mtype) != PRO_TK_NO_ERROR) {
      of << "Failed to Get Current Model Type ";
      } else {
      if (mtype == PRO_MDL_ASSEMBLY) {
      of << " - Assembly Model\n";
      aflag = 1;
      } else if  (mtype == PRO_MDL_PART) {
      of << " - Part Model\n";
      aflag = 0;
      } else {
      of << " - UnSupported Model Type " << mtype << "\n";
      return 0;
      }
      }*/

  status = ProMechmodeldataAlloc(&mdata);
  print_status("ProMechModeldataAlloc");
  status = ProMechmodeldataTypeSet(mdata, PRO_MECH_MODEL_SOLID);
  print_status("ProMechmodeldataTypeSet");
  status = ProMechmodeldataPreserveidsSet(mdata, PRO_B_TRUE);
  print_status("ProMechmodeldataPreserveidsSet");

  temp1Model = ProMdlToSolid(tempModel);
  
  status = ProMechanicaEnter(temp1Model,PRO_MECHANICA_STRUCTURE,PRO_B_TRUE);
  print_status("ProMechanicaEnter");

  status = ProMdlDisplay(tempModel);  //required for wf4-M030 and above or ProMechmodelGet will fail
  status = ProMechmodelGet(temp1Model, mdata, &mModel, &mStatus);
  print_status("ProMechmodelGet");

  if(status == PRO_TK_NO_ERROR) {
    // Now get the number of edges in the model
    status = ProMechmodelSolidVisit(mModel,solidIter,NULL,NULL);
    fprintf(logfile, "ProMechModel total faces: %d\n", numberOfFaces);
    //======================================================================



    status=ProMdlCurrentGet(&model); print_status("ProMdlCurrentGet");
  }
  status = ProMechanicaLeave();
  print_status("ProMechanicaLeave");
  fclose(logfile);

  ProInfoWindowDisplay(L"log.txt", NULL, NULL);

  return(0);
}


void user_terminate()
{
}
